{-# LANGUAGE RecordWildCards, NamedFieldPuns, TemplateHaskell, DeriveDataTypeable, DeriveGeneric, MultiParamTypeClasses, FlexibleInstances #-}

{- |

Module: BaselMachine.stopSignal

Description : Data structures representing different kinds of Signals

Copyright   : © 2019 W. Garrett Mitchener

License     : GPL-3

Maintainer  : garrett.mitchener@gmail.com

Stability   : experimental

Portability : non-portable (GHC, POSIX)


-}

module BaselMachine.Signal where

import GHC.Generics

import Control.DeepSeq

import Data.Aeson.TH
import Data.Data
import Data.Typeable

import BaselMachine.AesonUtil

class SingalInformation signal where

  -- | Is the signal ambient (part of the background or
  -- environment)?
  isAmbient :: signal -> Bool

  -- | Is the signal something produced by an agent?
  isAgentProduced :: signal -> Bool


class BroadcastSignal location signal where
  strengthAt ::
    signal -- ^ original state
    -> location -- ^ point of origin
    -> Int -- ^ elapsed time
    -> location -- ^ point of measurement
    -> Double -- ^ strength at location after elapsed time



-- | This is a simple broadcast that holds a value at
-- full strength everywhere at all times.

data NullSignal = NullSignal
  deriving (Eq, Ord, Show, Read, Data, Typeable, Generic)

instance NFData NullSignal
$(deriveJSON aesonOptions ''NullSignal)

instance BroadcastSignal location NullSignal where
  strengthAt NullSignal _ _ _ = 1.0
