{-# LANGUAGE FlexibleInstances, ParallelListComp #-}

{-|

Module: Digiostelium.DBStorable

Description: Class for types that can be stored in a database

Copyright: &#xA9; 2019 W. Garrett Mitchener

License     : GPL-3

Maintainer  : garrett.mitchener@gmail.com

Stability   : experimental

Portability : non-portable (GHC, POSIX)

I like to use objects like this
@
data TableNames = Address | PhoneNumber ...
@
for the names of tables.

-}


module Digiostelium.DBStorable (
  DBStorable(..)
  ) where

import Digiostelium.Database


class DBStorable t where

  dbStore :: (Show tableName, Ord tableName) =>
             t -- ^ the object
             -> String -- ^ key under which to store it
             -> tableName -- ^ name of the table
             -> DBMonad tableName ()

  dbLoad :: (Show tableName, Ord tableName) =>
             String -- ^ key to load
             -> tableName -- ^ tabel to look in
             -> DBMonad tableName t
