{-# LANGUAGE RecordWildCards, NamedFieldPuns, DeriveDataTypeable, DeriveGeneric, TemplateHaskell, StandaloneDeriving, FlexibleContexts, UndecidableInstances  #-}

{-|

Module: Digiostelium.PopulationSpec

Description: Data structures for specifying the parts of a run
             that only pertain to the population structure, not
             the problem being solved

Copyright   : © W. Garrett Mitchener 2019

License     : GPL-3

Maintainer  : garrett.mitchener@gmail.com

Stability   : experimental

Portability : non-portable (GHC, POSIX)


-}

module Digiostelium.PopulationSpec where

-- import GHC.Generics

-- import Data.Aeson
import Data.Data
-- import Data.Typeable

import Digiostelium.Database
import Digiostelium.Evolution
import Digiostelium.Genome


data PopulationSpec gs =
  PopulationSpec {
    populationSize :: Int,
    databaseSpec :: DatabaseSpec,
    genomeSpec :: gs,
    selectionSpec :: SelectionSpec,
    alignmentSpec :: AlignmentSpec gs
    }

deriving instance (Read gs, Read (AlignmentSpec gs)) => Read (PopulationSpec gs)
deriving instance (Show gs, Show (AlignmentSpec gs)) => Show (PopulationSpec gs)
deriving instance (Data gs, Data (AlignmentSpec gs)) => Data (PopulationSpec gs)
deriving instance Typeable PopulationSpec
