{-|

Module: Data.BitUtil

Description: Various conversions between 'Int', 'Integer', and
other ways to represent bits using standard Haskell types.

Copyright: © W. Garrett Mitchener 2017

License: GPL-3

Maintainer: garrett.mitchener@gmail.com

Stability: experimental

Portability: Portable

-}

module Data.BitUtil where

import Data.Bits
import Numeric


-- | Take a list of indices and returns an 'Int' where exactly
-- those bits are set.

fromBitIndices :: (Num t, Bits t) => [Int] -> t
fromBitIndices [] = 0
fromBitIndices (n:ns) = bit n .|. fromBitIndices ns



-- | Turn a list of bits back into a 'Bits'-instance object

fromBits :: (Num t, Bits t) => [Bool] -> t
fromBits [] = 0
fromBits (b:bs) = (if b then 1 else 0) .|. shiftL (fromBits bs) 1


-- | Produce a number of bits from an 'Integer' or other 'Bits'
-- instance

toBits :: Bits t => Int -> t -> [Bool]
toBits nBits n = map (testBit n) [0..nBits - 1]


-- | Convert a list of 'Bool' into an 'Integer' and show it in
-- hexadecimal.

showAsHex :: [Bool] -> String
showAsHex bits =
  let n :: Integer
      n = fromBits bits
  in showHex n ""


-- | Convert 'True' to the character '1' and 'False' to the
-- character '0' throughout a list.

showAs01s :: [Bool] -> [Char]
showAs01s bs =
  map (\b -> if b then '1' else '0') bs
