#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

#include "TR.h"
#include "CachingSFMT.h"

#define ALIGN 16

const char * cachingSFMT_vcid = "$Id$";

/* DSFMT_N is the minimum size for the vectorized array-filling
 * routines and it's pretty large. */

#define CACHE_SIZE (2*(int)DSFMT_N)

/**
 * Refill the cache of pseudorandom doubles
 *
 * @param csfmt csfmt internal state
 */
static void csfmt_refill(csfmt_t *csfmt)
{
  TR("%p", csfmt);
  TR("%p", csfmt->dsfmt);
  TR("%p", csfmt->cache);

  dsfmt_fill_array_close_open(csfmt->dsfmt, csfmt->cache,
                              CACHE_SIZE);
  /* and move the nextNumber pointer to its first element */
  csfmt->nextNumber = csfmt->cache;

  TR("%p", csfmt);
  TR("%p", csfmt->dsfmt);
  TR("%p", csfmt->cache);
}

/**
 * Refill the cache of pseudorandom doubles if needed
 *
 * @param csfmt csfmt internal state
 */
static void csfmt_maybe_refill(csfmt_t *csfmt)
{
  if(csfmt->nextNumber >= csfmt->cache + CACHE_SIZE)
    {
      csfmt_refill(csfmt);
    }
}

csfmt_t * csfmt_allocate()
{
  csfmt_t * csfmt;
  dsfmt_t * dsfmt;
  double * cache;
  int err;

  TR("%d", (int)DSFMT_N);
  TR("%d", (int)DSFMT_N32);
  TR("%d", (int)DSFMT_N64);
  TR("%d", (int)CACHE_SIZE);

  err = posix_memalign((void **)&dsfmt, ALIGN, sizeof(dsfmt_t));
  assert(!err);
  
  err = posix_memalign((void **)&cache, ALIGN, sizeof(double) * CACHE_SIZE);
  assert(!err);
 
  /*
    err = posix_memalign((void **)&csfmt, 256, sizeof(csfmt_t));
    assert(!err); */
  
  csfmt = malloc(sizeof(csfmt_t));
  assert(csfmt);

  csfmt->dsfmt = dsfmt;
  csfmt->cache = cache;

  TR("%p", csfmt);
  TR("%p", csfmt->dsfmt);
  TR("%p", csfmt->cache);

  return csfmt;
}

csfmt_t * csfmt_new(uint32_t seed)
{
  csfmt_t * csfmt;

  csfmt = csfmt_allocate();

  TR("%d", seed);
  TR("%p", csfmt);
  TR("%p", csfmt->dsfmt);
  TR("%p", csfmt->cache);

  dsfmt_init_gen_rand(csfmt->dsfmt, seed);

  TR("%p", csfmt);
  TR("%p", csfmt->dsfmt);
  TR("%p", csfmt->cache);

  csfmt_refill(csfmt);

  return csfmt;
}

csfmt_t * csfmt_newFromWords(uint32_t * seeds, int nWords)
{
  csfmt_t * csfmt;

  csfmt = csfmt_allocate();
  dsfmt_init_by_array(csfmt->dsfmt, seeds, nWords);
  csfmt_refill(csfmt);

  return csfmt;
}


void csfmt_finalize(csfmt_t * csfmt)
{
  TR("%p", csfmt);
  TR("%p", csfmt->dsfmt);
  TR("%p", csfmt->cache);

  if(csfmt)
    {
      free(csfmt->cache);
      free(csfmt->dsfmt);
      free(csfmt);
    }
}

/**
 * Generate an unsigned 32-bit integer.
 *
 * @param csfmt csfmt internal state
 * @return a pseudorandom double in the range [0,1)
 */

double csfmt_genrand_close_open(csfmt_t *csfmt)
{
  double x;

  x = *csfmt->nextNumber;
  ++(csfmt->nextNumber);
  csfmt_maybe_refill(csfmt);
  return x;
}

/**
 * Generate an unsigned 32-bit integer.
 *
 * @param csfmt csfmt state
 * @return 32 bit integer pseudorandom number
 */

uint32_t csfmt_genrand_uint32(csfmt_t *csfmt)
{
  double x;
  x = csfmt_genrand_close_open(csfmt);
  return (uint32_t) (x * 4294967296.0);
}
