/** 
 * @file CachingSFMT.h 
 *
 * @brief a wrapping of the dSFMT pseudorandom number generator,
 * where the generator is paired with a cache of random doubles
 * that were generated in parallel and emitted one at a time.
 *
 * @author Garrett Mitchener (College of Charleston)
 *
 * Copyright (C) 2010 Garrett Mitchener.
 * All rights reserved.
 *
 */

#if !defined(CACHING_SFMT_H)
#define CACHING_SFMT_H

#if !defined(HAVE_SSE2)
#define HAVE_SSE2
#endif
#if !defined(DSFMT_MEXP)
#define DSFMT_MEXP 19937
#endif


#include "dSFMT.h"

struct CSFMT_T {
  dsfmt_t * dsfmt;
  double * nextNumber;
  double * cache;
};
typedef struct CSFMT_T csfmt_t;

csfmt_t * csfmt_allocate(void);
csfmt_t * csfmt_new(uint32_t seed);
csfmt_t * csfmt_newFromWords(uint32_t * seeds, int nWords);
double csfmt_genrand_close_open(csfmt_t *csfmt);
uint32_t csfmt_genrand_uint32(csfmt_t *csfmt);
void csfmt_finalize(csfmt_t * csfmt);

#endif
