{-|

Module: WGM.FileUtil

Description: A few utilities for use with files.

Copyright: © W. Garrett Mitchener 2019

License: GPL-3

Maintainer: garrett.mitchener@gmail.com

Stability: experimental

Portability: POSIX

-}


module WGM.FileUtil (
  doesFileExistAndNonempty
  )where

import Control.Monad
import Control.Monad.Trans
import System.IO
import System.Directory

import System.Posix.Files


-- | Return whether the given 'FilePath' refers to an existing
-- non-empty file.  Less portable but probably safer than
-- 'doesFileExistAndNonemptyPortable'

doesFileExistAndNonempty :: MonadIO m => FilePath -> m Bool
doesFileExistAndNonempty path = liftIO $ do
  fileExists <- doesFileExist path
  if fileExists
    then do
    fileStatus <- getFileStatus path
    return (isRegularFile fileStatus
            && fileSize fileStatus > 0)
    else
    return False



-- | Return whether the given 'FilePath' refers to an existing
-- non-empty file.  This is mostly portable but will probably
-- raise an exception if called on a directory.

doesFileExistAndNonemptyPortable ::
  MonadIO m => FilePath -> m Bool
doesFileExistAndNonemptyPortable fileName = liftIO $ do
  exists <- doesFileExist fileName
  if exists
    then do
    n <- withFile fileName ReadMode $ \h -> do
      hFileSize h
    return (n > 0)
    else
    return False
